<?php
/**
 * Split.tg API 客户端 - PHP版本
 * 
 * 此脚本提供了与Split.tg API直接交互的功能，用于购买Telegram Premium和Stars
 * 无需浏览器，适用于无界面Linux服务器环境
 * 
 * 使用方法:
 * php split-api-direct.php --telegram-user=username --months=3 --token=YOUR_ACCESS_TOKEN
 */

// 设置错误报告
error_reporting(E_ALL);
ini_set('display_errors', 1);

// 设置默认时区
date_default_timezone_set('UTC');

/**
 * 日志类
 */
class Logger {
    private $logFile;
    
    /**
     * 构造函数
     * @param string $logPath 日志路径
     */
    public function __construct($logPath) {
        // 确保日志目录存在
        if (!file_exists($logPath)) {
            mkdir($logPath, 0755, true);
        }
        
        // 创建日志文件
        $this->logFile = $logPath . '/split-api-' . date('Y-m-d\TH-i-s') . '.log';
    }
    
    /**
     * 记录日志
     * @param string $message 日志消息
     * @param string $level 日志级别
     */
    public function log($message, $level = 'INFO') {
        $timestamp = date('Y-m-d\TH:i:s\Z');
        $logMessage = "[$timestamp] [$level] $message";
        
        echo $logMessage . PHP_EOL;
        file_put_contents($this->logFile, $logMessage . PHP_EOL, FILE_APPEND);
    }
}

/**
 * HTTP客户端类
 */
class HttpClient {
    private $timeout;
    private $logger;
    private $baseUrl;
    private $accessToken;
    
    /**
     * 构造函数
     * @param Logger $logger 日志对象
     * @param string $baseUrl API基础URL
     * @param string $accessToken 访问令牌
     * @param int $timeout 超时时间(毫秒)
     */
    public function __construct($logger, $baseUrl, $accessToken, $timeout) {
        $this->logger = $logger;
        $this->baseUrl = rtrim($baseUrl, '/');
        $this->accessToken = $accessToken;
        $this->timeout = $timeout / 1000; // 转换为秒
    }
    
    /**
     * 发送GET请求
     * @param string $endpoint API端点
     * @param array $params 查询参数
     * @return array 响应数据
     */
    public function get($endpoint, $params = []) {
        $url = $this->baseUrl . $endpoint;
        if (!empty($params)) {
            $url .= '?' . http_build_query($params);
        }
        
        $this->logger->log("发送GET请求: $url");
        
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, $this->timeout);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $this->getHeaders());
        
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        
        if (curl_errno($ch)) {
            $error = curl_error($ch);
            curl_close($ch);
            $this->logger->log("GET请求失败: $error", 'ERROR');
            throw new Exception("GET请求失败: $error");
        }
        
        curl_close($ch);
        
        $responseData = json_decode($response, true);
        
        if ($httpCode >= 400) {
            $errorMessage = isset($responseData['error_message']) ? $responseData['error_message'] : "HTTP错误: $httpCode";
            $this->logger->log("API错误: $errorMessage", 'ERROR');
            throw new Exception("API错误: $errorMessage");
        }
        
        return $responseData;
    }
    
    /**
     * 发送POST请求
     * @param string $endpoint API端点
     * @param array $data 请求数据
     * @return array 响应数据
     */
    public function post($endpoint, $data) {
        $url = $this->baseUrl . $endpoint;
        
        $this->logger->log("发送POST请求: $url");
        
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_TIMEOUT, $this->timeout);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($ch, CURLOPT_HTTPHEADER, $this->getHeaders());
        
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        
        if (curl_errno($ch)) {
            $error = curl_error($ch);
            curl_close($ch);
            $this->logger->log("POST请求失败: $error", 'ERROR');
            throw new Exception("POST请求失败: $error");
        }
        
        curl_close($ch);
        
        $responseData = json_decode($response, true);
        
        if ($httpCode >= 400) {
            $errorMessage = isset($responseData['error_message']) ? $responseData['error_message'] : "HTTP错误: $httpCode";
            $this->logger->log("API错误: $errorMessage", 'ERROR');
            throw new Exception("API错误: $errorMessage");
        }
        
        return $responseData;
    }
    
    /**
     * 获取请求头
     * @return array 请求头数组
     */
    private function getHeaders() {
        $headers = [
            'Content-Type: application/json',
            'Accept: application/json',
            'User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36'
        ];
        
        if (!empty($this->accessToken)) {
            $headers[] = "Authorization: Bearer {$this->accessToken}";
        }
        
        return $headers;
    }
}

/**
 * 重试工具类
 */
class RetryUtil {
    private $logger;
    
    /**
     * 构造函数
     * @param Logger $logger 日志对象
     */
    public function __construct($logger) {
        $this->logger = $logger;
    }
    
    /**
     * 重试函数
     * @param callable $fn 要重试的函数
     * @param int $maxRetries 最大重试次数
     * @param int $delay 重试延迟(毫秒)
     * @param string $operationName 操作名称(用于日志)
     * @return mixed 函数结果
     */
    public function retry($fn, $maxRetries, $delay, $operationName) {
        $lastException = null;
        
        for ($attempt = 1; $attempt <= $maxRetries; $attempt++) {
            try {
                return $fn();
            } catch (Exception $e) {
                $lastException = $e;
                if ($attempt < $maxRetries) {
                    $this->logger->log("{$operationName}失败(尝试 $attempt/$maxRetries): {$e->getMessage()}，" . ($delay/1000) . "秒后重试...", 'WARN');
                    usleep($delay * 1000); // 微秒
                }
            }
        }
        
        throw $lastException;
    }
}

/**
 * Split API客户端类
 */
class SplitApiClient {
    private $httpClient;
    private $logger;
    private $retryUtil;
    private $retryCount;
    private $retryDelay;
    
    /**
     * 构造函数
     * @param HttpClient $httpClient HTTP客户端
     * @param Logger $logger 日志对象
     * @param RetryUtil $retryUtil 重试工具
     * @param int $retryCount 重试次数
     * @param int $retryDelay 重试延迟(毫秒)
     */
    public function __construct($httpClient, $logger, $retryUtil, $retryCount, $retryDelay) {
        $this->httpClient = $httpClient;
        $this->logger = $logger;
        $this->retryUtil = $retryUtil;
        $this->retryCount = $retryCount;
        $this->retryDelay = $retryDelay;
    }
    
    /**
     * 创建用户
     * @param string|null $inviter 邀请人钱包地址
     * @return array 用户信息
     */
    public function createUser($inviter = null) {
        return $this->retryUtil->retry(function() use ($inviter) {
            try {
                $data = [];
                if ($inviter) {
                    $data['inviter'] = $inviter;
                }
                
                $response = $this->httpClient->post('/user/create', $data);
                return isset($response['message']['user']) ? $response['message']['user'] : $response;
            } catch (Exception $e) {
                $this->logger->log("创建用户失败: {$e->getMessage()}", 'ERROR');
                throw $e;
            }
        }, $this->retryCount, $this->retryDelay, '创建用户');
    }
    
    /**
     * 获取当前用户信息
     * @return array 用户信息
     */
    public function getMe() {
        return $this->retryUtil->retry(function() {
            try {
                $response = $this->httpClient->get('/user/me');
                return isset($response['message']['user']) ? $response['message']['user'] : $response;
            } catch (Exception $e) {
                $this->logger->log("获取用户信息失败: {$e->getMessage()}", 'ERROR');
                throw $e;
            }
        }, $this->retryCount, $this->retryDelay, '获取用户信息');
    }
    
    /**
     * 解析Premium接收者
     * @param string $username 用户名
     * @return array 接收者信息
     */
    public function resolvePremiumRecipient($username) {
        return $this->retryUtil->retry(function() use ($username) {
            try {
                $response = $this->httpClient->post('/resolve/premium/recipient', [
                    'username' => $username
                ]);
                return isset($response['message']['recipient']) ? $response['message']['recipient'] : $response;
            } catch (Exception $e) {
                $this->logger->log("解析Premium接收者失败: {$e->getMessage()}", 'ERROR');
                throw $e;
            }
        }, $this->retryCount, $this->retryDelay, '解析Premium接收者');
    }
    
    /**
     * 解析Stars接收者
     * @param string $username 用户名
     * @return array 接收者信息
     */
    public function resolveStarsRecipient($username) {
        return $this->retryUtil->retry(function() use ($username) {
            try {
                $response = $this->httpClient->post('/resolve/stars/recipient', [
                    'username' => $username
                ]);
                return isset($response['message']['recipient']) ? $response['message']['recipient'] : $response;
            } catch (Exception $e) {
                $this->logger->log("解析Stars接收者失败: {$e->getMessage()}", 'ERROR');
                throw $e;
            }
        }, $this->retryCount, $this->retryDelay, '解析Stars接收者');
    }
    
    /**
     * 购买Premium
     * @param string $recipient 接收者地址
     * @param int $months 月数
     * @param string $username 用户名
     * @return array 交易信息
     */
    public function buyPremium($recipient, $months, $username) {
        return $this->retryUtil->retry(function() use ($recipient, $months, $username) {
            try {
                $response = $this->httpClient->post('/buy/premium', [
                    'recipient' => $recipient,
                    'months' => $months,
                    'username' => $username,
                    'payment_method' => 'TON_CONNECT'
                ]);
                return isset($response['message']['transaction']) ? $response['message']['transaction'] : $response;
            } catch (Exception $e) {
                $this->logger->log("购买Premium失败: {$e->getMessage()}", 'ERROR');
                throw $e;
            }
        }, $this->retryCount, $this->retryDelay, '购买Premium');
    }
    
    /**
     * 购买Stars
     * @param string $recipient 接收者地址
     * @param int $quantity 数量
     * @param string $username 用户名
     * @return array 交易信息
     */
    public function buyStars($recipient, $quantity, $username) {
        return $this->retryUtil->retry(function() use ($recipient, $quantity, $username) {
            try {
                $response = $this->httpClient->post('/buy/stars', [
                    'recipient' => $recipient,
                    'quantity' => $quantity,
                    'username' => $username,
                    'payment_method' => 'TON_CONNECT'
                ]);
                return isset($response['message']['transaction']) ? $response['message']['transaction'] : $response;
            } catch (Exception $e) {
                $this->logger->log("购买Stars失败: {$e->getMessage()}", 'ERROR');
                throw $e;
            }
        }, $this->retryCount, $this->retryDelay, '购买Stars');
    }
    
    /**
     * 获取购买费用
     * @return float 费用
     */
    public function getBuyFee() {
        return $this->retryUtil->retry(function() {
            try {
                $response = $this->httpClient->get('/get/buy/fee');
                return isset($response['message']['fee']) ? (float)$response['message']['fee'] : 0;
            } catch (Exception $e) {
                $this->logger->log("获取购买费用失败: {$e->getMessage()}", 'ERROR');
                throw $e;
            }
        }, $this->retryCount, $this->retryDelay, '获取购买费用');
    }
    
    /**
     * 获取TON汇率
     * @return float 汇率
     */
    public function getTonRate() {
        return $this->retryUtil->retry(function() {
            try {
                $response = $this->httpClient->get('/get/ton/rate');
                return isset($response['message']['rate']) ? (float)$response['message']['rate'] : 0;
            } catch (Exception $e) {
                $this->logger->log("获取TON汇率失败: {$e->getMessage()}", 'ERROR');
                throw $e;
            }
        }, $this->retryCount, $this->retryDelay, '获取TON汇率');
    }
}

/**
 * 主应用类
 */
class SplitApiApp {
    private $logger;
    private $httpClient;
    private $retryUtil;
    private $splitApiClient;
    private $params;
    
    /**
     * 构造函数
     * @param array $params 参数
     */
    public function __construct($params) {
        $this->params = $params;
        
        // 初始化日志
        $logPath = $params['log-path'] ?? __DIR__ . '/logs';
        $this->logger = new Logger($logPath);
        
        // 检查访问令牌
        $accessToken = $params['token'] ?? getenv('SPLIT_API_TOKEN') ?: '';
        if (empty($accessToken)) {
            $this->logger->log('错误: 未提供访问令牌，请通过--token参数或SPLIT_API_TOKEN环境变量设置', 'ERROR');
            exit(1);
        }
        
        // 初始化HTTP客户端
        $baseUrl = $params['api-url'] ?? getenv('SPLIT_API_URL') ?: 'https://split.tg/api';
        $timeout = (int)($params['timeout'] ?? getenv('TIMEOUT') ?: 30000);
        $this->httpClient = new HttpClient($this->logger, $baseUrl, $accessToken, $timeout);
        
        // 初始化重试工具
        $this->retryUtil = new RetryUtil($this->logger);
        
        // 初始化Split API客户端
        $retryCount = (int)($params['retry-count'] ?? getenv('RETRY_COUNT') ?: 3);
        $retryDelay = (int)($params['retry-delay'] ?? getenv('RETRY_DELAY') ?: 3000);
        $this->splitApiClient = new SplitApiClient($this->httpClient, $this->logger, $this->retryUtil, $retryCount, $retryDelay);
    }
    
    /**
     * 运行应用
     * @return int 退出码
     */
    public function run() {
        // 检查必要参数
        $telegramUser = $this->params['telegram-user'] ?? $this->params['telegram-id'] ?? null;
        if (!$telegramUser) {
            $this->logger->log('错误: 缺少必要参数 --telegram-user 或 --telegram-id', 'ERROR');
            return 1;
        }
        
        // 格式化用户名 (确保@符号正确)
        $formattedUser = $telegramUser;
        // 如果是纯数字，则视为ID
        if (preg_match('/^\d+$/', $telegramUser)) {
            // 保持原样，这是ID
        } 
        // 如果不是以@开头的用户名，添加@
        elseif (substr($telegramUser, 0, 1) !== '@') {
            $formattedUser = '@' . $telegramUser;
        }
        
        // 设置默认值
        $months = (int)($this->params['months'] ?? 3);
        $stars = (int)($this->params['stars'] ?? 0);
        $dryRun = ($this->params['dry-run'] ?? 'false') === 'true'; // 干运行模式，不实际执行购买
        
        // 确定操作类型
        $operationType = 'premium';
        if (isset($this->params['stars']) && $stars > 0) {
            $operationType = 'stars';
        }
        
        if ($operationType === 'premium') {
            $this->logger->log("开始为用户 $formattedUser 购买 $months 个月的Premium会员...");
        } else {
            $this->logger->log("开始为用户 $formattedUser 购买 $stars 个Stars...");
        }
        
        // 干运行模式
        if ($dryRun) {
            $this->logger->log('干运行模式: 不会实际执行购买操作');
            if ($operationType === 'premium') {
                $this->logger->log("将为用户 $formattedUser 购买 $months 个月的Premium会员");
            } else {
                $this->logger->log("将为用户 $formattedUser 购买 $stars 个Stars");
            }
            $this->logger->log('干运行完成');
            return 0;
        }
        
        $success = false;
        
        try {
            // 获取当前用户信息
            $this->logger->log('正在获取当前用户信息...');
            $user = $this->splitApiClient->getMe();
            $this->logger->log("当前用户: {$user['id']}");
            
            // 获取TON汇率
            $this->logger->log('正在获取TON汇率...');
            $tonRate = $this->splitApiClient->getTonRate();
            $this->logger->log("当前TON汇率: $tonRate");
            
            // 获取购买费用
            $this->logger->log('正在获取购买费用...');
            $buyFee = $this->splitApiClient->getBuyFee();
            $this->logger->log("购买费用: $buyFee");
            
            if ($operationType === 'premium') {
                // 解析Premium接收者
                $this->logger->log("正在解析Premium接收者: $formattedUser");
                $recipient = $this->splitApiClient->resolvePremiumRecipient($formattedUser);
                $this->logger->log("接收者地址: {$recipient['address']}");
                
                // 购买Premium
                $this->logger->log("正在为用户 $formattedUser 购买 $months 个月的Premium会员...");
                $transaction = $this->splitApiClient->buyPremium($recipient['address'], $months, $formattedUser);
                $this->logger->log("交易创建成功，交易ID: {$transaction['id']}");
                $this->logger->log("交易状态: {$transaction['status']}");
                
                if ($transaction['status'] === 'completed' || $transaction['status'] === 'pending') {
                    $this->logger->log("成功为用户 $formattedUser 购买了 $months 个月的Premium会员");
                    $success = true;
                } else {
                    throw new Exception("交易状态异常: {$transaction['status']}");
                }
            } else {
                // 解析Stars接收者
                $this->logger->log("正在解析Stars接收者: $formattedUser");
                $recipient = $this->splitApiClient->resolveStarsRecipient($formattedUser);
                $this->logger->log("接收者地址: {$recipient['address']}");
                
                // 购买Stars
                $this->logger->log("正在为用户 $formattedUser 购买 $stars 个Stars...");
                $transaction = $this->splitApiClient->buyStars($recipient['address'], $stars, $formattedUser);
                $this->logger->log("交易创建成功，交易ID: {$transaction['id']}");
                $this->logger->log("交易状态: {$transaction['status']}");
                
                if ($transaction['status'] === 'completed' || $transaction['status'] === 'pending') {
                    $this->logger->log("成功为用户 $formattedUser 购买了 $stars 个Stars");
                    $success = true;
                } else {
                    throw new Exception("交易状态异常: {$transaction['status']}");
                }
            }
        } catch (Exception $e) {
            $this->logger->log("操作失败: {$e->getMessage()}", 'ERROR');
        }
        
        return $success ? 0 : 1;
    }
}

// 加载环境变量
if (file_exists(__DIR__ . '/.env')) {
    $envLines = file(__DIR__ . '/.env', FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
    foreach ($envLines as $line) {
        // 跳过注释
        if (strpos(trim($line), '#') === 0) {
            continue;
        }
        
        $line = trim($line);
        if (!empty($line)) {
            list($name, $value) = explode('=', $line, 2);
            $name = trim($name);
            $value = trim($value);
            
            // 移除引号
            if (preg_match('/^([\'"])(.*)\1$/', $value, $matches)) {
                $value = $matches[2];
            }
            
            putenv("$name=$value");
        }
    }
}

// 解析命令行参数
$params = [];
$args = array_slice($argv, 1);

foreach ($args as $arg) {
    if (strpos($arg, '--') === 0) {
        $parts = explode('=', substr($arg, 2), 2);
        $key = $parts[0];
        $value = isset($parts[1]) ? $parts[1] : 'true';
        $params[$key] = $value;
    }
}

// 运行应用
$app = new SplitApiApp($params);
$exitCode = $app->run();

exit($exitCode);
